@@ -1,8 +1,9 @@
 from collections import OrderedDict
+import re
 import pprint
 import sys
 
-from typing import Dict, List, Mapping, MutableMapping, Optional, Set, Tuple
+from typing import Dict, List, Mapping, Optional, Pattern, Set, Tuple
 
 from mypy import defaults
 
@@ -42,10 +43,11 @@ class Options:
         "always_false",
         "strict_optional",
         "disallow_untyped_decorators",
+        "local_partial_types",
     }
 
     OPTIONS_AFFECTING_CACHE = ((PER_MODULE_OPTIONS |
-                                {"quick_and_dirty", "platform"})
+                                {"quick_and_dirty", "platform", "bazel"})
                                - {"debug_cache"})
 
     def __init__(self) -> None:
@@ -63,6 +65,8 @@ class Options:
         self.custom_typeshed_dir = None  # type: Optional[str]
         self.mypy_path = []  # type: List[str]
         self.report_dirs = {}  # type: Dict[str, str]
+        # Show errors in PEP 561 packages/site-packages modules
+        self.no_silence_site_packages = False
         self.ignore_missing_imports = False
         self.follow_imports = 'normal'  # normal|silent|skip|error
         # Whether to respect the follow_imports setting even for stub files.
@@ -166,8 +170,8 @@ class Options:
         self.plugins = []  # type: List[str]
 
         # Per-module options (raw)
-        pm_opts = OrderedDict()  # type: OrderedDict[str, Dict[str, object]]
-        self.per_module_options = pm_opts
+        self.per_module_options = OrderedDict()  # type: OrderedDict[str, Dict[str, object]]
+        self.glob_options = []  # type: List[Tuple[str, Pattern[str]]]
         self.unused_configs = set()  # type: Set[str]
 
         # -- development options --
@@ -185,12 +189,26 @@ class Options:
         self.use_builtins_fixtures = False
 
         # -- experimental options --
-        self.shadow_file = None  # type: Optional[Tuple[str, str]]
+        self.shadow_file = None  # type: Optional[List[Tuple[str, str]]]
         self.show_column_numbers = False  # type: bool
         self.dump_graph = False
         self.dump_deps = False
         # If True, partial types can't span a module top level and a function
         self.local_partial_types = False
+        # Some behaviors are changed when using Bazel (https://bazel.build).
+        self.bazel = False
+        # If True, export inferred types for all expressions as BuildResult.types
+        self.export_types = False
+        # List of package roots -- directories under these are packages even
+        # if they don't have __init__.py.
+        self.package_root = []  # type: List[str]
+        self.cache_map = {}  # type: Dict[str, Tuple[str, str]]
+
+    def snapshot(self) -> object:
+        """Produce a comparable snapshot of this Option"""
+        d = dict(self.__dict__)
+        del d['per_module_cache']
+        return d
 
     def __eq__(self, other: object) -> bool:
         return self.__class__ == other.__class__ and self.__dict__ == other.__dict__
@@ -199,31 +217,58 @@ class Options:
         return not self == other
 
     def __repr__(self) -> str:
-        d = dict(self.__dict__)
-        del d['per_module_cache']
-        return 'Options({})'.format(pprint.pformat(d))
+        return 'Options({})'.format(pprint.pformat(self.snapshot()))
+
+    def apply_changes(self, changes: Dict[str, object]) -> 'Options':
+        new_options = Options()
+        new_options.__dict__.update(self.__dict__)
+        new_options.__dict__.update(changes)
+        return new_options
 
     def build_per_module_cache(self) -> None:
         self.per_module_cache = {}
-        # Since configs inherit from glob configs above them in the hierarchy,
+
+        # Config precedence is as follows:
+        #  1. Concrete section names: foo.bar.baz
+        #  2. "Unstructured" glob patterns: foo.*.baz, in the order
+        #     they appear in the file (last wins)
+        #  3. "Well-structured" wildcard patterns: foo.bar.*, in specificity order.
+
+        # Since structured configs inherit from structured configs above them in the hierarchy,
         # we need to process per-module configs in a careful order.
-        # We have to process foo.* before foo.bar.* before foo.bar.
-        # To do this, process all glob configs before non-glob configs and
+        # We have to process foo.* before foo.bar.* before foo.bar,
+        # and we need to apply *.bar to foo.bar but not to foo.bar.*.
+        # To do this, process all well-structured glob configs before non-glob configs and
         # exploit the fact that foo.* sorts earlier ASCIIbetically (unicodebetically?)
         # than foo.bar.*.
-        keys = (sorted(k for k in self.per_module_options.keys() if k.endswith('.*')) +
-                [k for k in self.per_module_options.keys() if not k.endswith('.*')])
-        for key in keys:
+        # (A section being "processed last" results in its config "winning".)
+        # Unstructured glob configs are stored and are all checked for each module.
+        unstructured_glob_keys = [k for k in self.per_module_options.keys()
+                                  if '*' in k[:-1]]
+        structured_keys = [k for k in self.per_module_options.keys()
+                           if '*' not in k[:-1]]
+        wildcards = sorted(k for k in structured_keys if k.endswith('.*'))
+        concrete = [k for k in structured_keys if not k.endswith('.*')]
+
+        for glob in unstructured_glob_keys:
+            self.glob_options.append((glob, self.compile_glob(glob)))
+
+        # We (for ease of implementation) treat unstructured glob
+        # sections as used if any real modules use them or if any
+        # concrete config sections use them. This means we need to
+        # track which get used while constructing.
+        self.unused_configs = set(unstructured_glob_keys)
+
+        for key in wildcards + concrete:
             # Find what the options for this key would be, just based
             # on inheriting from parent configs.
             options = self.clone_for_module(key)
             # And then update it with its per-module options.
-            new_options = Options()
-            new_options.__dict__.update(options.__dict__)
-            new_options.__dict__.update(self.per_module_options[key])
-            self.per_module_cache[key] = new_options
+            self.per_module_cache[key] = options.apply_changes(self.per_module_options[key])
 
-        self.unused_configs = set(keys)
+        # Add the more structured sections into unused configs, since
+        # they only count as used if actually used by a real module.
+        self.unused_configs.update(structured_keys)
 
     def clone_for_module(self, module: str) -> 'Options':
         """Create an Options object that incorporates per-module options.
@@ -245,18 +290,38 @@ class Options:
         # in that order, looking for an entry.
         # This is technically quadratic in the length of the path, but module paths
         # don't actually get all that long.
+        options = self
         path = module.split('.')
         for i in range(len(path), 0, -1):
             key = '.'.join(path[:i] + ['*'])
             if key in self.per_module_cache:
                 self.unused_configs.discard(key)
-                return self.per_module_cache[key]
+                options = self.per_module_cache[key]
+                break
+
+        # OK and *now* we need to look for unstructured glob matches.
+        # We only do this for concrete modules, not structured wildcards.
+        if not module.endswith('.*'):
+            for key, pattern in self.glob_options:
+                if pattern.match(module):
+                    self.unused_configs.discard(key)
+                    options = options.apply_changes(self.per_module_options[key])
 
         # We could update the cache to directly point to modules once
         # they have been looked up, but in testing this made things
         # slower and not faster, so we don't bother.
 
-        return self
+        return options
+
+    def compile_glob(self, s: str) -> Pattern[str]:
+        # Compile one of the glob patterns to a regex so that '.*' can
+        # match *zero or more* module sections. This means we compile
+        # '.*' into '(\..*)?'.
+        parts = s.split('.')
+        expr = re.escape(parts[0]) if parts[0] != '*' else '.*'
+        for part in parts[1:]:
+            expr += re.escape('.' + part) if part != '*' else '(\..*)?'
+        return re.compile(expr + '\\Z')
 
     def select_options_affecting_cache(self) -> Mapping[str, object]:
         return {opt: getattr(self, opt) for opt in self.OPTIONS_AFFECTING_CACHE}
