@@ -10,8 +10,7 @@ belongs to a module involved in an import loop.
 """
 
 from collections import OrderedDict
-from contextlib import contextmanager
-from typing import Dict, List, Callable, Optional, Union, Set, cast, Tuple, Iterator
+from typing import Dict, List, Callable, Optional, Union, cast, Tuple
 
 from mypy import messages, experiments
 from mypy.nodes import (
@@ -19,7 +18,7 @@ from mypy.nodes import (
     Block, TypedDictExpr, NamedTupleExpr, AssignmentStmt, IndexExpr, TypeAliasExpr, NameExpr,
     CallExpr, NewTypeExpr, ForStmt, WithStmt, CastExpr, TypeVarExpr, TypeApplication, Lvalue,
     TupleExpr, RevealExpr, SymbolTableNode, SymbolTable, Var, ARG_POS, OverloadedFuncDef,
-    MDEF,
+    MDEF, TypeAlias
 )
 from mypy.types import (
     Type, Instance, AnyType, TypeOfAny, CallableType, TupleType, TypeVarType, TypedDictType,
@@ -77,6 +76,7 @@ class SemanticAnalyzerPass3(TraverserVisitor, SemanticAnalyzerCoreInterface):
     def refresh_partial(self, node: Union[MypyFile, FuncItem, OverloadedFuncDef],
                         patches: List[Tuple[int, Callable[[], None]]]) -> None:
         """Refresh a stale target in fine-grained incremental mode."""
+        self.options = self.sem.options
         self.patches = patches
         if isinstance(node, MypyFile):
             self.recurse_into_functions = False
@@ -204,7 +204,7 @@ class SemanticAnalyzerPass3(TraverserVisitor, SemanticAnalyzerCoreInterface):
             sig = find_fixed_callable_return(dec.decorators[0])
             if sig:
                 # The outermost decorator always returns the same kind of function,
-                # so we know that this is the type of the decoratored function.
+                # so we know that this is the type of the decorated function.
                 orig_sig = function_type(dec.func, self.builtin_type('function'))
                 sig.name = orig_sig.items()[0].name
                 dec.var.type = sig
@@ -243,15 +243,8 @@ class SemanticAnalyzerPass3(TraverserVisitor, SemanticAnalyzerCoreInterface):
             if isinstance(analyzed, NamedTupleExpr):
                 self.analyze(analyzed.info.tuple_type, analyzed, warn=True)
                 self.analyze_info(analyzed.info)
-        # We need to pay additional attention to assignments that define a type alias.
-        # The resulting type is also stored in the 'type_override' attribute of
-        # the corresponding SymbolTableNode.
         if isinstance(s.lvalues[0], RefExpr) and isinstance(s.lvalues[0].node, Var):
             self.analyze(s.lvalues[0].node.type, s.lvalues[0].node)
-            if isinstance(s.lvalues[0], NameExpr):
-                node = self.sem.lookup(s.lvalues[0].name, s, suppress_errors=True)
-                if node:
-                    self.analyze(node.type_override, node)
         # Subclass attribute assignments with no type annotation should be
         # assumed to be classvar if overriding a declared classvar from the base
         # class.
@@ -288,8 +281,7 @@ class SemanticAnalyzerPass3(TraverserVisitor, SemanticAnalyzerCoreInterface):
 
     # Helpers
 
-    def perform_transform(self, node: Union[Node, SymbolTableNode],
-                          transform: Callable[[Type], Type]) -> None:
+    def perform_transform(self, node: Node, transform: Callable[[Type], Type]) -> None:
         """Apply transform to all types associated with node."""
         if isinstance(node, ForStmt):
             if node.index_type:
@@ -305,6 +297,8 @@ class SemanticAnalyzerPass3(TraverserVisitor, SemanticAnalyzerCoreInterface):
                              TypeAliasExpr, Var)):
             assert node.type, "Scheduled patch for non-existent type"
             node.type = transform(node.type)
+        if isinstance(node, TypeAlias):
+            node.target = transform(node.target)
         if isinstance(node, NewTypeExpr):
             assert node.old_type, "Scheduled patch for non-existent type"
             node.old_type = transform(node.old_type)
@@ -337,9 +331,6 @@ class SemanticAnalyzerPass3(TraverserVisitor, SemanticAnalyzerCoreInterface):
                                         transform(node.info.tuple_type))
         if isinstance(node, TypeApplication):
             node.types = [transform(t) for t in node.types]
-        if isinstance(node, SymbolTableNode):
-            assert node.type_override, "Scheduled patch for non-existent type"
-            node.type_override = transform(node.type_override)
         if isinstance(node, TypeInfo):
             for tvar in node.defn.type_vars:
                 if tvar.upper_bound:
@@ -370,7 +361,7 @@ class SemanticAnalyzerPass3(TraverserVisitor, SemanticAnalyzerCoreInterface):
             for item in lvalue.items:
                 self.transform_types_in_lvalue(item, transform)
 
-    def analyze(self, type: Optional[Type], node: Union[Node, SymbolTableNode],
+    def analyze(self, type: Optional[Type], node: Node,
                 warn: bool = False) -> None:
         # Recursive type warnings are only emitted on type definition 'node's, marked by 'warn'
         # Flags appeared during analysis of 'type' are collected in this dict.
@@ -378,7 +369,11 @@ class SemanticAnalyzerPass3(TraverserVisitor, SemanticAnalyzerCoreInterface):
         if type:
             analyzer = self.make_type_analyzer(indicator)
             type.accept(analyzer)
-            self.check_for_omitted_generics(type)
+            if not (isinstance(node, TypeAlias) and node.no_args):
+                # We skip bare type aliases like `A = List`, these
+                # are still valid. In contrast, use/expansion points
+                # like `x: A` will be flagged.
+                self.check_for_omitted_generics(type)
             self.generate_type_patches(node, indicator, warn)
             if analyzer.aliases_used:
                 target = self.scope.current_target()
@@ -399,8 +394,8 @@ class SemanticAnalyzerPass3(TraverserVisitor, SemanticAnalyzerCoreInterface):
     def analyze_symbol_table(self, names: SymbolTable) -> None:
         """Analyze types in symbol table nodes only (shallow)."""
         for node in names.values():
-            if node.type_override:
-                self.analyze(node.type_override, node)
+            if isinstance(node.node, TypeAlias):
+                self.analyze(node.node.target, node.node)
 
     def make_scoped_patch(self, fn: Callable[[], None]) -> Callable[[], None]:
         saved_scope = self.scope.save()
@@ -411,7 +406,7 @@ class SemanticAnalyzerPass3(TraverserVisitor, SemanticAnalyzerCoreInterface):
         return patch
 
     def generate_type_patches(self,
-                              node: Union[Node, SymbolTableNode],
+                              node: Node,
                               indicator: Dict[str, bool],
                               warn: bool) -> None:
         if indicator.get('forward') or indicator.get('synthetic'):
